# Detectors

[solution_name] uses detectors to find and extract dependencies from all supported package managers.

Each package manager ecosystem is assigned a detector type. Each detector type may have multiple methods used to extract dependencies.

## Detector Search

Detectors first check to see if they apply to your project by looking for hints such as files that exist in your project directory
or properties you have set.

By default, detectors only search the project directory. In some cases, such as when your project contains sub-projects,
or when package manager files reside in sub-directories, you may need to tell [solution_name] to search sub-directories
by increasing the detector search depth. For more information, refer to [detector search depth](../properties/configuration/paths.md#detector-search-depth).

Detectors then check that your environment is extractable, meaning you have all the relevant executables such as NPM or a Gradle wrapper, and all relevant downloads are present or available, such as the Docker or NuGet inspector.

Inspectors are used by detectors when the package manager requires an integration or embedded plugin to work. For example, Gradle uses an inspector as a plugin that executes a custom task. Most detectors do not require an inspector.

Finally, detectors perform their extraction to find your dependencies. This may require, but is not limited to, running executables, performing builds, parsing files, and communicating with web services.

## Build detectors versus buildless detectors

The recommended way to run [solution_name] is as a post-build step, so that it has access to both build artifacts and the build tools (package managers and others) used to build the project.
[solution_name]'s build detectors work in this environment and produce the most accurate results. By default, [solution_name] runs build detectors.

If you can't build your project, you may still be able to use [solution_name]'s buildless detectors. The results from buildless detectors may not be as accurate as the results from
build detectors, but buildless detectors can run without accessing the tools required to build the project. You can choose to run
buildless detectors using the [buildless mode property](../properties/configuration/general.md#buildless-mode).

The following tables show which detectors run in the default (build) mode, and which detectors run in buildless mode. There is some overlap across the two lists.

## Build detectors

By default, [solution_name] requires that you are able to build the project you are scanning. This allows [solution_name] to get the most accurate results possible.
Build detectors can communicate with package managers and run commands for example: *mvn dependency:tree*, and/or inspectors; such as the [Gradle inspector](inspectors.md#gradle-inspector) to derive dependency information.
Not all build detectors run external commands or communicate with external systems but all build detectors return accurate results.

Each applicable detector's required executables (as shown in the following table) must be present and findable on your system. [solution_name] uses
the system PATH to find executables. In some cases as an alternative to the system PATH, the location of an executable can be provided through a property.

|Type|Name|Language|Forge|Requirements
|---|---|---|---|---|
|BITBAKE | Bitbake |various|YOCTO | Properties: Package names <br /><br /> File: build env script.<br /><br /> Executable: bash|
|CARGO | Cargo |Rust|crates | Files: Cargo.lock, Cargo.toml|
|CARTHAGE | Carthage |various|GitHub | Files: Cartfile, Cartfile.resolved|
|CLANG | Clang |C or C++|Derived from the Linux distribution. | File: compile_commands.json. <br /><br /> Executable: Linux package manager.|
|COCOAPODS | Pod Lock |Objective C|COCOAPODS and NPMJS | Files: Podfile.lock|
|CONAN | Conan CLI |C/C++|conan | Files: conanfile.txt or conanfile.py. <br /><br /> Executable: conan.|
|CONAN | Conan Lockfile |C/C++|conan | Files: conan.lock.|
|CONDA | Conda Cli |Python|Anaconda | File: environment.yml. <br /><br /> Executable: conda.|
|CPAN | Cpan Cli |Perl|CPAN | File: Makefile.PL. <br /><br /> Executable: cpan.|
|CRAN | Packrat Lock |R|CRAN | File: packrat.lock.|
|DART | Dart Pub Deps |Dart|Dart | Files: pubspec.yaml, pubspec.lock.<br/><br/>Executable: dart, flutter|
|GIT | Git |various|N/A | Directory: .git. <br /><br /> Executable: git. <br /><br /> OR <br /><br /> Files: .git/config, .git/HEAD.|
|GO_DEP | Go Lock |Golang|GitHub | File: Gopkg.lock.|
|GO_GRADLE | Go Gradle |Golang|GitHub | File: gogradle.lock.|
|GO_MOD | Go Mod Cli |Golang|Go Modules | File: go.mod.<br/><br/>Executable: go.|
|GO_VENDOR | Go Vendor |Golang|GitHub | File: vendor/vendor.json.|
|GO_VNDR | Go Vndr |Golang|GitHub | File: vendor.conf.|
|GRADLE | Gradle Inspector |various|Maven Central | File: build.gradle or build.gradle.kts.<br/><br/>Executable: gradlew or gradle.|
|HEX | Rebar |Erlang|Hex | File: rebar.config.<br/><br/>Executable: rebar3.|
|LERNA | Lerna |Node JS|npmjs | File: package.json, and one of the following: package-lock.json, npm-shrinkwrap.json, yarn.lock.|
|MAVEN | Maven Pom |various|Maven Central | File: pom.xml.<br/><br/>Executable: mvnw or mvn.|
|MAVEN | Maven Wrapper |various|Maven Central | File: pom.groovy.<br/><br/>Executable: mvnw or mvn.|
|NPM | Npm Cli |Node JS|npmjs | Files: node_modules, package.json. <br /><br /> Executable: npm.|
|NPM | Package Lock |Node JS|npmjs | File: package-lock.json. Optionally for better results: package.json also.|
|NPM | Shrinkwrap |Node JS|npmjs | File: npm-shrinkwrap.json. Optionally for better results: package.json also.|
|NUGET | Project |C#|NuGet.org | File: a project file with one of the following extensions: .csproj, .fsproj, .vbproj, .asaproj, .dcproj, .shproj, .ccproj, .sfproj, .njsproj, .vcxproj, .vcproj, .xproj, .pyproj, .hiveproj, .pigproj, .jsproj, .usqlproj, .deployproj, .msbuildproj, .sqlproj, .dbproj, .rproj|
|NUGET | Solution |C#|NuGet.org | File: a solution file with a .sln extension.|
|PACKAGIST | Composer |PHP|Packagist.org | Files: composer.lock, composer.json.|
|PEAR | Pear |PHP|Pear | Files: package.xml.<br/><br/> Executable: pear.|
|PIP | Pip Env |Python|PyPi | Files: Pipfile or Pipfile.lock.<br/><br/>Executables: python or python3, and pipenv.|
|PIP | Pip Inspector |Python|Pypi | A setup.py file, or one or more requirements.txt files.<br/><br/> Executables: python and pip, or python3 and pip3.|
|PNPM | Pnpm Lock |Node JS|npmjs | Files: pnpm-lock.yaml and package.json.|
|POETRY | Poetry |Python|pypi | Files: Poetry.lock, pyproject.toml|
|RUBYGEMS | Gemlock |Ruby|RubyGems | File: Gemfile.lock.|
|SBT | Sbt |Scala|Maven Central | File: build.sbt. Plugin: Dependency Graph|
|SWIFT | Swift |Swift|Swift.org | File: Package.swift.<br/><br/> Executables: swift.|
|YARN | Yarn Lock |Node JS|npmjs | Files: yarn.lock and package.json.|

## Buildless detectors

When in buildless mode, only detectors that do not communicate with external systems are run. Typically these detectors just parse available package manager files such as *pom.xml* to derive dependency information.
It is generally not recommended to run in buildless mode as the results are not guaranteed to be accurate - for example [solution_name] may report dependencies with fuzzy versions.
However if the project can not be built, buildless mode still allows some insight into the dependencies of a project.

|Type|Name|Language|Forge|Requirements
|---|---|---|---|---|
|CARGO | Cargo |Rust|crates | Files: Cargo.lock, Cargo.toml|
|CARTHAGE | Carthage |various|GitHub | Files: Cartfile, Cartfile.resolved|
|COCOAPODS | Pod Lock |Objective C|COCOAPODS and NPMJS | Files: Podfile.lock|
|DART | Dart Pub Spec |Dart|Dart | Files: pubspec.yaml, pubspec.lock.|
|GIT | Git Parse |various|N/A | Files: .git/config, .git/HEAD.|
|GO_DEP | Go Lock |Golang|GitHub | File: Gopkg.lock.|
|GO_GRADLE | Go Gradle |Golang|GitHub | File: gogradle.lock.|
|GO_VENDOR | Go Vendor |Golang|GitHub | File: vendor/vendor.json.|
|GO_VNDR | Go Vndr |Golang|GitHub | File: vendor.conf.|
|GRADLE | Gradle Project Inspector |various|Maven Central | File: build.gradle|
|MAVEN | Maven Pom Parse |various|Maven Central | File: pom.xml.|
|MAVEN | Maven Project Inspector |various|Maven Central | File: pom.xml.|
|NPM | Package Json Parse |Node JS|npmjs | File: package.json.|
|NPM | Package Lock |Node JS|npmjs | File: package-lock.json. Optionally for better results: package.json also.|
|NPM | Shrinkwrap |Node JS|npmjs | File: npm-shrinkwrap.json. Optionally for better results: package.json also.|
|NUGET | NuGet Project Inspector |C#|NuGet.org | File: a project file with one of the following extensions: .csproj, .sln|
|PACKAGIST | Composer |PHP|Packagist.org | Files: composer.lock, composer.json.|
|PACKAGIST | Packrat Lock |R|CRAN | File: packrat.lock.|
|PNPM | Pnpm Lock |Node JS|npmjs | Files: pnpm-lock.yaml and package.json.|
|POETRY | Poetry |Python|pypi | Files: Poetry.lock, pyproject.toml|
|RUBYGEMS | Gemlock |Ruby|RubyGems | File: Gemfile.lock.|
|RUBYGEMS | Gemspec |Ruby|RubyGems | File: A gemspec file (with .gemspec extension).|
|YARN | Yarn Lock |Node JS|npmjs | Files: yarn.lock and package.json.|
